using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Workflow.Runtime.Tracking;

namespace CustomTrackingService
{
    /// <summary>
    /// This class implements the TrackingService class which provides the basic interface
    /// between a tracking service and the runtime tracking infrastructure.
    /// </summary>
	class MyTrackingService : TrackingService
	{
        // TrackingProfile tells the workflow runtime which events and pieces of data we are
        // interested in , in our case , we will load a trackingprofile from
        // an xml file and store it in this variable.
        private TrackingProfile ourOnlyProfile;

        /// <summary>
        /// Returns the tracking profile for the specified workflow Type. 
        /// </summary>
        /// <param name="workflowInstanceId"></param>
        /// <returns></returns>
        protected override TrackingProfile GetProfile(
            Guid workflowInstanceId)
        {
            if(ourOnlyProfile == null)
            {

               using ( StreamReader reader = new StreamReader("profile.xml") )
               {
                   TrackingProfileSerializer serialize = new TrackingProfileSerializer();
                   
                   // deserializing the profile stores in xml file 
                   ourOnlyProfile = serialize.Deserialize(reader);
                   
                   reader.Close();
               }

            }

            return ourOnlyProfile;
        }

        /// <summary>
        /// returns the tracking profile, qualified by version, for the specified workflow Type. 
        /// We in our case return same profile in all cases
        /// </summary>
        /// <param name="workflowType"></param>
        /// <param name="profileVersionId"></param>
        /// <returns></returns>
        protected override TrackingProfile GetProfile(
            Type workflowType, Version profileVersionId)
        {
            return GetProfile(Guid.Empty);
        }

        /// <summary>
        /// retrieves a new tracking profile for the specified workflow instance
        /// if the tracking profile has changed since it was last loaded.
        /// The profile never gets updated in our case.
        /// </summary>
        /// <param name="workflowType"></param>
        /// <param name="workflowInstanceId"></param>
        /// <param name="profile"></param>
        /// <returns></returns>
        protected override bool TryReloadProfile(
            Type workflowType, Guid workflowInstanceId, out TrackingProfile profile)
        {
            profile =null;
            //in our case , the profile never gets out of date.
            return false;
        }

        protected override bool TryGetProfile(
            Type workflowType, out TrackingProfile profile)
        {
            profile = GetProfile(Guid.Empty);
            return true;
        }

        /// <summary>
        /// If a TrackingService is present, the runtime tracking infrastructure 
        /// calls GetTrackingChannel to get a tracking channel for a workflow instance.
        /// </summary>
        /// <param name="parameters"></param>
        /// <returns></returns>
         protected override TrackingChannel GetTrackingChannel(
            TrackingParameters parameters)
        {
           return new MyTrackingChannel();
        }
	}
}
